@@ -22,7 +22,7 @@ class _Folds:
 
 # Class to hold and train many models (CB, XGB and LGB) on the same dataset
 class GBMFitter:
-    def __init__(self, cb_data=[], xgb_data=[], lgb_data=[], cv_strategy=None, cv_groups=None, use_rounders=False, metrics=['rmse']):
+    def __init__(self, cb_data=[], xgb_data=[], lgb_data=[], cv_strategy=None, cv_groups=None, use_rounders=False, metrics=['rmse'], verbose=1, logfile=None):
         """Generates models in a CV manner for all 3 libraries algorithms
 
         Inputs:
@@ -48,6 +48,9 @@ class GBMFitter:
         else:
             self.cv = cv_strategy
 
+        self.verbose = verbose
+        self.logfile = logfile
+
 
     def _get_categorical_index(self, df, cat_cols):
         cat_features_index = np.where(df.columns.isin(cat_cols))[0].tolist()
@@ -88,12 +91,12 @@ class GBMFitter:
         if not self.training_data[library]:
             return
 
-        print("Training " + library + " models")
+        self.log("Training " + library + " models")
         start = time.time()
         for model in self.training_data[library]:
             # Perform Feature Selection
             if feature_selection > 0:
-                print("\tFeature Selection for " + library + " models")
+                self.log("\tFeature Selection for " + library + " models")
                 fselection = select_features(library)
                 fselection.fit(self._train(library, model, X, y, categorical)[0], feature_selection)
                 self.fselection[library] = fselection
@@ -101,11 +104,11 @@ class GBMFitter:
 
             # Training for CV
             if self.cv is not None:
-                print("\tTraining with " + str(self.cv.get_n_splits()) + " folds")
+                self.log("\tTraining with " + str(self.cv.get_n_splits()) + " folds")
                 # Perform CV
                 y_pred = np.zeros(X_data.shape[0])
                 for f, (train_index, val_index) in enumerate(self.cv.split(X_data, y, self.cv_groups)):
-                    print("\t\tTraining fold {} ".format(f))
+                    self.log("\t\tTraining fold {} ".format(f))
                     y_pred[val_index] = self._train(library,
                                                     model,
                                                     train=(X_data.iloc[train_index], y.iloc[train_index]),
@@ -117,7 +120,7 @@ class GBMFitter:
                         value = mt.mean_squared_error(y, y_pred, squared=False)
                     else:
                         value = metric(y, y_pred)
-                    print("\t\tOOF Validation Metric: {:.4f}, total time elapsed {}".format(value, str(round(time.time() - start, 2))))
+                    self.log("\t\tOOF Validation Metric: {:.4f}, total time elapsed {}".format(value, str(round(time.time() - start, 2))))
 
                 # save OOF results
                 self.oof[library].append(y_pred)
@@ -229,7 +232,7 @@ class GBMFitter:
         # f1 = f1_score(y_val, y_pred_val, average='macro')
 
         # print("\t\t\tTrain Accuracy: {:.4f}, Train F1: {:.4f}, Val Accuracy: {:.4f}, Val F1: {:.4f},  elapsed {}".format( acc_train, f1_train, acc, f1, str(time.time() - start)) )
-        print("\t\t\tTrain Metric: {:.4f}, OOF Val Metric: {:.4f}, elapsed {}".format(train_metric, val_metric,
+        self.log("\t\t\tTrain Metric: {:.4f}, OOF Val Metric: {:.4f}, elapsed {}".format(train_metric, val_metric,
                                                                                   str(round(time.time() - start, 2))))
 
         # Store model and rounder if needed
@@ -311,6 +314,17 @@ class GBMFitter:
 
         return df
 
+    def log(self, message):
+        """ Log training ouput into console and file
+            input:
+                message: message to be logged
+        """
+        if self.verbose > 0:
+            print(message)
+        if self.logfile:
+            with open(self.logfile, 'a+') as logger:
+                logger.write(f'{message}\n')
+
 
 def select_features(library):
     if library == 'CB':
